<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoEscala extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrturno        = $itens['NRTURNO']             ??  null;
        $nrmotivoescala = $itens['NRMOTIVOESCALA']      ??  null;
        $nrescalatrabm  = $itens['NRESCALATRABM']       ??  null;
        $dtiniescala    = isset($itens['DTINIESCALA'])  ?   DateUtil::getDataDeString(substr($itens['DTINIESCALA'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtcompetencia  = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO,true);
        $nrorg          = Utils::getParamters()->getNrorg();
        $nrvinculom     = $this->getSolicitacao()->getNrvinculom();
        
        $result = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTEESCALA)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtiniescala" => $dtiniescala));

        if($result) {
            throw new \Exception("Já existe uma alteração de escala para este vinculo nesta data");
        }
        
        $this->atualizaAlteEscala(
            $nrorg,
            $nrvinculom,
            DateUtil::subtraiIntervalo($dtiniescala,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),
            DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),
            Utils::getParamters()->getCdoperador()
        );
        
        $gpeAlteescala = Repositories::GPE_ALTEESCALA;
        $alteEscala = new $gpeAlteescala;
        $alteEscala->setNrorg($nrorg);
        $alteEscala->setNrvinculom($nrvinculom);
        $alteEscala->setNrturno($nrturno);
        $alteEscala->setNrmotivoescala($nrmotivoescala);
        $alteEscala->setNrescalatrabm($nrescalatrabm);
        $alteEscala->setDtiniescala($dtiniescala);
        $alteEscala->setDtinclusao(DateUtil::getDataAtual(false));
        $alteEscala->setNrorginclusao(Utils::getParamters()->getNrorg());
        $alteEscala->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $alteEscala->setNralteescala(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_ALTEESCALA', 1, 12, true));
        Utils::getEntityManager()->persist($alteEscala);
        Utils::getEntityManager()->flush();
    }
    
    public function rejeitar() {}
    
    public function atualizaAlteEscala($nrorg,$nrvinculom, $dtfimescala,$dtatual,$cdoperador) {
        $sql = <<<SQL
            UPDATE GPE_ALTEESCALA 
            SET DTFIMESCALA = :P_DTFIMESCALA,
            DTULTATU = :P_DTATUAL,
            NRORGULTATU = :P_NRORG,
            CDOPERULTATU = :P_CDOPERADOR
            WHERE DTINIESCALA = (
                SELECT MAX(DTINIESCALA) 
                FROM GPE_ALTEESCALA 
                WHERE NRORG = :P_NRORG 
                AND NRVINCULOM = :P_NRVINCULOM 
                AND DTFIMESCALA IS NULL
            )
            AND NRORG = :P_NRORG 
            AND NRVINCULOM = :P_NRVINCULOM 
            AND DTINIESCALA >= :P_DTFIMESCALA
            AND DTFIMESCALA IS NULL
SQL;
        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"           => $nrorg,
                "P_DTFIMESCALA"     => $dtfimescala,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTATUAL"         => $dtatual,
                "P_CDOPERADOR"      => $cdoperador
            )
        );
    }
}