<?php

namespace Util\Mail\SendEmail;

use Util\Repositories;
use Util\Messages;

use Util\Mail\Mail;
use Util\Mail\SMTP;

use Util\Mail\SendEmail\MailDataSource;

class SendEmail {
    
    private const AVAILABLE_TEMPLATES = array(
        'TEKNISA' => 'TEKNISA_TEMPLATE'
    );
    
    public static function returnParamsSmtp($nrorg, $lote = false){
        
        $userName = null;
        $passWord = null;
        $host = null;
        $port = null;
        $SMTPSecure = null;
        
        $paramOrg = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->retornaParamOrg($nrorg);
        
        if($paramOrg[0]){
           $paramOrg = $paramOrg[0];
           $userName = $paramOrg["DSEMAILSMTP"] ? $paramOrg["DSEMAILSMTP"] : $userName;
           $passWord = $paramOrg["DSSENHASMTP"] ? $paramOrg["DSSENHASMTP"] : $passWord;
           $host = $paramOrg["DSHOSTSMTP"] ? $paramOrg["DSHOSTSMTP"] : $host;
           $port = $paramOrg["DSPORTASMTP"] ? $paramOrg["DSPORTASMTP"] : $port;
           $SMTPSecure = $paramOrg["IDSMTPSECURE"] ? $paramOrg["IDSMTPSECURE"] : $SMTPSecure;
        }
        
        if(!$userName || !$passWord || !$host || !$port){
            if($lote){	 
                return false;	 
            }
            throw new \Exception(Messages::MAIL_ERR_PARAM);
        }
        
        $params = array(
            'host'          => $host,
            'username'      => $userName,
            'password'      => $passWord,
            'port'          => $port,
            'from'          => $userName,
            'SMTPSecure'    => $SMTPSecure
        );
        
        $smtp = new SMTP($params);
        $mail = new Mail($smtp);
        
        return $mail;    
    }
    
    /**
     * Rotina de envio de email
     * 
     * @param string $templateName
     * @param string $email
     * @param string $subject
     * @param string $nmdestinatario
     * @param string $nmremetente
     * @param string $nrorg
     * @param string $nrestruturam
     * @param Array  $content
     * @param string $attachment = null (file path)
     * @param bool   $validaEmailTabelaOper = true
     */
    public static function sendEmail($templateName, $email, $subject, $nmdestinatario, $nmremetente, $nrorg, $nrestruturam, $content, $attachment = null, $validaEmailTabelaOper = true) {
        try {
            if($validaEmailTabelaOper){
                // Valida se o email é existente
                self::validaEmailExistente($email);
            }
            
            if(!$email){
                throw new \Exception(strtr(Repositories::MAIL_DEST_NOT_FOUND, array('@' => $nmdestinatario)));
            }
            // Seta logo estrutura
            $logoEstrutura = self::getUrlLogoByNrestruturam($nrorg, $nrestruturam);
            $content = array_merge($content, array('URL_LOGO_CLIENTE' => $logoEstrutura));
            
            switch ($templateName) {
                case self::AVAILABLE_TEMPLATES['TEKNISA']:
                    $templateTeknisa = new \Util\Mail\Templates\Teknisa\TemplateTeknisa(\Setforms2\Shared\Registry::getEntityManager());
                    $content = $templateTeknisa->setValuesTemplate($content);
                    $mail = self::returnParamsSmtp($nrorg);
                    $return = $mail->newEmail($email, $content, $subject, $nmdestinatario, $nmremetente, null, $attachment, true);
                    return $return;
                    break;
                
                default:
                    throw new \Exception(Messages::MAIL_TEMPLATE_NOT_FOUND);
                    break;
            }
            
            return true;
        } catch(\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    public static function validaEmailExistente($dsemailoper){

        if (is_numeric($dsemailoper)) {
            $this->retornaEmailByCdoperador();
        }

        try {
            $result = MailDataSource::getOperadorByDsemailoper(\Setforms2\Shared\Registry::getEntityManager(), $dsemailoper);

            if (count($result) > 0) {
                return true;
            } else {
                throw new \Exception(Messages::MAIL_DEST_NOT_FOUND_OPERADOR);
            }
        }
        catch (Exception $e) {
            throw new exception ($e->getMessage());
        }

    }

    public static function getUrlLogoByNrestruturam($nrorg, $nrestruturam) {

        if (!empty($nrestruturam)) {
            
            $result = MailDataSource::getUrlLogoByNrestruturam(\Setforms2\Shared\Registry::getEntityManager(), $nrorg, $nrestruturam);

            if (count($result) > 0)
                return $result[0]['DSURLLOGOESTRUTURA'];
            else 
                return null;
        } else {
            return null;
        }
    }
}